/*********************************************************************
 
 Beispielprogramm fuer das BLIT2008-Board
 (http://bralug.de/wiki/BLIT2008-Board)
  
 Blit-Board goes to onewire... ;-)
 
 Hier eine einfache Implementierung des 1wire-Protokolls fuer das
 Blit-Board.
 
 Die Kernroutinen fuer 1wire sind der Etherrape-Firmware [1] entnommen
 und nur noch fuer das Blit-Board angepasst worden.

 Als weitere Einschraenkung ist unbedingt zu nennen, dass in diesem
 Programm nur ein 1wire-Slave am Bus erkannt und abgefragt wird. Sind
 mehrere Clients gewuenscht, muss noch etwas getan werden.
 
 Es werden nur DS18x20 und DS1822 (beides Temperaturfuehler) erkannt.
 Als 1wire-Anschluss wurde PIN PB3 des ATmega8 (befindet sich auf
 dem Stecker des ISB auf dem Blit-Board) verwendet.

 Uwe Berger; 2009
 ---------
 Have fun!

 [1] http://www.lochraster.org

 
********************************************************************/

#ifndef F_CPU
	#define F_CPU 12000000
#endif

#include <avr/interrupt.h>
#include <stdlib.h>

#include "i2clcd.h"
#include "onewire.h"

#define PRELOAD 55000

volatile int i = 0;

//1wire-Zeugs
int16_t ret;
struct ow_rom_code_t rom;
struct ow_temp_scratchpad_t sp;



/***************************************************************************/
// ISR Timer1-Ueberlauf
ISR(TIMER1_OVF_vect)
{
	TCNT1 = PRELOAD;			// Preloadwert setzen
	i++;
}


/***************************************************************************/
int16_t parse_onewire_list(uint8_t *ptr)
{
    int16_t ret;

    if (ow_global.lock == 0) {
        ow_global.lock = 1;
        /* disable interrupts */
        uint8_t sreg = SREG;
        cli();
        ret = ow_search_rom_first();
        /* re-enable interrupts */
        SREG = sreg;
        if (ret <= 0) {
            return 0;
        }
    } else {
        /* disable interrupts */
        uint8_t sreg = SREG;
        cli();
        ret = ow_search_rom_next();
        SREG = sreg;
    }

    if (ret == 1) {
		rom.raw = ow_global.current_rom.raw;
        /* set return value that the parser has to be called again */
        if (ret > 0) ret = -ret - 10;
        return ret;
    } else if (ret == 0) {
        ow_global.lock = 0;
        return 0;
    }
    return -1;
}


/***************************************************************************/
uint8_t ntohex(uint8_t val)
{
	// nibble to hex
	if (val > 9){
		return val + 'A' - 10;
	}
	else{
	return val + '0';
	}
}

/***************************************************************************/
void itohex(uint8_t val,char *str)
{
	// byte to hex
	*str++ = ntohex(val >> 4);
	*str = ntohex(val & 0x0f);
}

/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
int main(void)
{
	int old_i = 0;
	char s[10];
	uint16_t temp;
	uint8_t j;
	
	// Timer initialisieren...
	TCNT1   = PRELOAD;			// Preloadwert setzen
	TCCR1B |= (1 << CS02) | (1 << CS00);	// Prescaler Timer1 auf 1024 stellen
	TIMSK  |= (1 << TOIE1);			// Timer 1 Overflow Interrupt enable
	sei(); 					// Interrupts einschalten

	lcd_wait_ms(100);

	i2c_init();
	lcd_init();
	onewire_init();

	lcd_wait_ms(100);

	lcd_command(LCD_CLEAR);
	lcd_backlight(ON);
	lcd_printlc(1,1,(unsigned char *)"-->BLIT-Board<--");
	lcd_printlc(2,1,(unsigned char *)"Uwe Berger; 2008");
	lcd_wait_ms(1000);
	lcd_command(LCD_CLEAR);
	lcd_printlc(2,3,(unsigned char *)".  C");
	lcd_putcharlr(2, 5, 223); // Grad-Zeichen

	// Achtung, hier wird nur EIN 1wire-Slave ermittelt und
	// auch der nur in der Schleife unten zyklisch ausgelesen!!!
	// Wenn mehrere Busteilnehmer da sind, funktioniert das 
	// so nicht.
	// Um das richtig zu machen, wird man hier (und auch weiter unten)
	// eine Schleife und ein Array bemuehen muessen...
	//
	parse_onewire_list(&rom);
	//gefundene 1wire-ID ausgeben
	for (j=0; j<8; j++) {
		itohex(rom.bytewise[j], s);
		lcd_printlr(1, j*2+1, (unsigned char *)s);
	}
		
	while (true){

		if (old_i != i) {
			old_i = i;
			itoa(i, s, 10);
			lcd_printlr(2, 10, (unsigned char *)s);

			if (ow_temp_sensor(&rom)) {

				// Interrupt ausschalten
				uint8_t sreg = SREG;
				cli();
				// Slave anschubsen
				ret = ow_temp_start_convert(&rom, 1);
				// Slave auslesen
				ret = ow_temp_read_scratchpad(&rom, &sp);
				// Interrupt wieder einschalten
				SREG = sreg;
				// es gibt verschiedene DS1820...
				temp = ow_temp_normalize(&rom, &sp);
				itoa(HI8(temp), s, 10);
				lcd_printlr(2, 1, (unsigned char *)s);
				itoa(HI8(((temp & 0x00ff) * 10) + 0x80), s, 10);
				lcd_printlr(2, 4, (unsigned char *)s);
			}
		}
	}					
}
